/*******************************************************************************
 *******************************************************************************
 ** Core serial bus routines                                                  **
 *******************************************************************************
 *******************************************************************************/

	; VIA registers

via1_pa2 = $911f		; VIA 1 port A (no handshake)
					; bit	function
					; ---	--------
					;  7	serial ATN out
					;  1	serial DATA in
					;  0	serial CLK in
via2_t2ch = $9129		; VIA 2 timer 2 high-order counter
via2_pcr = $912c		; VIA 2 peripheral control register
					; bit	function
					; ---	--------
					; 765	CB2 serial DATA out
					; 321	CA2 serial CLK out
via2_ifr = $912d		; VIA 2 interrupt flag register
					; bit	function	cleared by
					; ---	--------	----------
					;  5	T2 interrupt	read T2C_l, write T2C_h


	; private zero page locations (re-used KERNAL tape locations)

jdev	= $96			; JiffyDOS device flag (b7)
pcr_mask = $9c			; VIA PCR bitmask
lo_nib	= $9e			; low nibble of receive byte
hi_nib	= $9f			; high nibble of receive byte


/*******************************************************************************
 * Send string to secondary address                                            *
 *                                                                             *
 * Command a serial device to listen and give a string of bytes in FNADR to a  *
 * secondary address in .Y (must include bus command e.g. $60).                *
 *                                                                             *
 * Returns: Cb = 0, success                                                    *
 *******************************************************************************/

	subroutine

ser_str:
	lda	fa		; get device number
	jsr	listen		; command device to LISTEN
	tya			; copy secondary address to .A
	jsr	second		; target message to secondary address
	lda	status		; get I/O status byte
	bmi	.dvntp		; device not present

	jsr	ser_fname	; send string
	jsr	unlsn		; command serial bus to UNLISTEN
	clc			; flag ok
	rts

.dvntp:
	pla			; dump calling address low byte
	pla			; dump calling address high byte
	jmp	fe_dvntp	; do device not present error and return


/*******************************************************************************
 * Send string to serial bus                                                   *
 *                                                                             *
 * Write FNLEN bytes of the string pointed to by FNADR to a serial device      *
 *******************************************************************************/

	subroutine

ser_fname:
	ldy	#0		; clear index
.loop:
	lda	(fnadr),y	; get string byte
	jsr	ciout		; output byte on serial bus
	iny			; increment index
	cpy	fnlen		; compare with string length
	bne	.loop		; loop until done

	rts


/*******************************************************************************
 * Close serial channel                                                        *
 *                                                                             *
 * Command secondary address in .Y to close (must include bus command          *
 * e.g. $e0).                                                                  *
 *******************************************************************************/

	subroutine

ser_close:
	lda	fa		; get device number
	jsr	listen		; command a serial bus device to LISTEN
	tya			; copy secondary address to .A
	jsr	second		; send secondary address after LISTEN
	jmp	unlsn		; command serial bus to UNLISTEN


/*******************************************************************************
 * Command a serial bus device to TALK                                         *
 *                                                                             *
 * Command device in .A to send data on the bus. Usually followed by TKSA.     *
 *******************************************************************************/

	subroutine

talk:
	ora	#$40		; OR device with the TALK command
	.byte	$2c		; make next instruction BIT


/*******************************************************************************
 * Command a serial bus device to LISTEN                                       *
 *                                                                             *
 * Command device in .A to receive data on the bus. Usually followed by SECOND.*
 *******************************************************************************/

listen:
	ora	#$20		; OR device with the LISTEN command
	jsr	rspause		; wait for RS-232 to be idle
	lsr	jdev		; clear JiffyDOS flag


/*******************************************************************************
 * Send command byte on serial bus                                             *
 *******************************************************************************/

cmd_send:
	bit	c3p0		; test deferred character flag
	bpl	.no_data	; branch if no deferred character

	; final data byte to send

	sec			; flag EOI
	ror	pcntr		; rotate into serial EOI flag
	pha			; save device command
	jsr	srsend		; Tx byte on serial bus
	pla			; restore device command
	lsr	c3p0		; clear deferred character flag
	lsr	pcntr		; clear serial EOI flag

.no_data:
	sta	bsour		; save as serial deferred character
	jsr	serout1		; set serial DATA high
	lda	via1_pa2	; get VIA 1 DRA, no handshake
	ora	#%10000000	; set serial ATN out low
	sta	via1_pa2	; set VIA 1 DRA, no handshake


/*******************************************************************************
 * Set CLK/DATA, wait and Tx byte on serial bus                                *
 *******************************************************************************/

ser_wait:
	jsr	srclklo		; set serial CLK low
	jsr	serout1		; set serial DATA high
	jsr	waitabit	; 1ms delay
	sei			; disable interrupts
	jsr	serout1		; set serial DATA high
	jsr	serget		; get serial CLK status
	lsr			; shift serial DATA to Cb
	bcs	.no_dev		; if DATA high do device not present

	jsr	srclkhi		; set serial CLK high

	; wait for serial DATA to rise

.wait_up:
	jsr	serget		; get serial CLK status
	lsr			; shift serial DATA to Cb
	bcc	.wait_up	; loop if DATA low

	; serial DATA is high, pull serial CLK low within 60us

	jsr	srclklo		; set serial CLK low

	; send the eight data bits, least significant bit first

	lda	#$07		; eight bits to do
	sta	cntdn		; set serial bus bit count

.next_bit:
	lda	via1_pa2	; get VIA 1 DRA, no handshake
	cmp	via1_pa2	; compare with self
	bne	.next_bit	; loop if changing

	lsr			; serial CLK to Cb
	lsr			; serial DATA to Cb
	bcc	.timeout	; if DATA high do timeout on serial bus

	ror	bsour		; rotate transmit byte into Cb
	bcs	.bit1		; branch if bit = 1

	jsr	serout0		; else set serial DATA low
	bne	.clk_hi		; branch always

.bit1:	
	jsr	serout1		; set serial DATA high

	; set serial CLK high for about 23us

.clk_hi:
	jsr	srclkhi		; {6} set serial CLK high
	nop			; {+2} delay
	dec	cntdn		; {+5} decrement bit count
	lda	via2_pcr	; {+4} get VIA 2 PCR
	and	#%11011111	; {+2} set CB2 low, serial DATA out high
	ora	#%00000010	; {+2} set CA2 high, serial CLK out low
	sta	via2_pcr	; {+4:22.6us} set VIA 2 PCR

	lda	cntdn		; get bit count
	beq	.check_jiffy	; final bit next, check for JiffyDOS
	bpl	.next_bit	; loop if not all done

	; all eight bits have been sent, the device should acknowledge by
	; pulling serial DATA low within 1ms

	lda	#$04		; set timeout for 1024 cycles, about 1ms
	sta	via2_t2ch	; set VIA 2 T2 counter high byte, clear interrupt flag

.wait_ack:
	lda	via2_ifr	; get VIA 2 IFR
	and	#%00100000	; mask T2 interrupt bit
	bne	.timeout	; if T2 interrupt do timeout on serial bus

	jsr	serget		; get serial CLK status
	lsr			; shift serial DATA to Cb
	bcs	.wait_ack	; if DATA high go wait some more

	cli			; enable interrupts
	rts

	; wait for serial DATA to go high, 0.3ms timeout

.check_jiffy:
	lda	#%00000010	; serial DATA in bitmask
	ldx	#$20		; set repeat counter

.wait_up3:
	bit	via1_pa2	; {+4} test VIA 1 DRA
	beq	.is_jiffy	; {+2/3} serial DATA high, JiffyDOS

	dex			; {+2} decrement counter
	bne	.wait_up3	; {+2/3} test bus again

	beq	.next_bit	; timeout, not a JiffyDOS device

	; wait for serial DATA to go low

.is_jiffy:
	bit	via1_pa2	; test VIA 1 DRA
	beq	.is_jiffy	; loop if serial DATA high

	sec
	ror	jdev		; set JiffyDOS flag
	bne	.next_bit	; send final bit, branch always


/*******************************************************************************
 * Report device not present                                                   *
 *******************************************************************************/

.no_dev:
	jmp	$eeb4		; set I/O status byte & complete command


/*******************************************************************************
 * Report bus timeout                                                          *
 *******************************************************************************/

.timeout:
	jmp	$eeb7		; set I/O status byte & complete command


/*******************************************************************************
 * Send secondary address after LISTEN                                         *
 *                                                                             *
 * Send command to secondary address in .A (must include $60).                 *
 *******************************************************************************/

second:
	sta	bsour		; save deferred byte
	jsr	ser_wait	; set CLK/DATA, wait & Tx byte on serial bus
	jmp	scatn		; set serial ATN high


/*******************************************************************************
 * Send secondary address after TALK                                           *
 *                                                                             *
 * Send command to secondary address in .A (must include $60).                 *
 *******************************************************************************/

tksa:
	sta	bsour		; save the secondary address byte to transmit
	jsr	ser_wait	; set CLK/DATA, wait & Tx byte on serial bus
	jmp	$eed3		; exchange transmit and receive roles


/*******************************************************************************
 * Command serial bus device to UNTALK                                         *
 *******************************************************************************/

untlk:
	jsr	srclklo		; set serial CLK low
	lda	via1_pa2	; get VIA 1 DRA, no handshake
	ora	#%10000000	; set serial ATN low
	sta	via1_pa2	; set VIA 1 DRA, no handshake

	lda	#$5f		; set the UNTALK command
	.byte	$2c		; make next instruction BIT


/*******************************************************************************
 * Command serial bus devices to UNLISTEN                                      *
 *******************************************************************************/

unlsn:
	lda	#$3f		; set the UNLISTEN command
	jsr	cmd_send	; send command byte on serial bus
	jmp	$ef09		; ATN high, delay, CLK high then DATA high


/*******************************************************************************
 * Send byte to listening serial bus device                                    *
 *                                                                             *
 * Transmit data in .A, EOI will be indicated for the byte before an UNLISTEN. *
 * Unlike the KERNAL routine .X is *not* preserved.                            *
 *******************************************************************************/

	subroutine

ciout:
	bit	c3p0		; test deferred character flag
	bpl	.first		; no deferred character, just store

	pha			; save byte
	jsr	srsend		; Tx byte on serial bus
	pla			; restore byte

.save:
	sta	bsour		; save as deferred byte
	clc			; flag ok
	rts

.first:
	sec			; set carry
	ror	c3p0		; shift into deferred character flag
	bne	.save		; save byte and exit, branch always


/*******************************************************************************
 * Send byte to listening serial bus device                                    *
 *                                                                             *
 * Transmit data in BSOUR using either the standard IEC protocol or the        *
 * JiffyDOS byte protocol.                                                     *
 *******************************************************************************/

	subroutine

.k_srsend:
	jmp	$ee49		; use normal KERNAL routine

srsend:
	bit	jdev		; test JiffyDOS flag
	bpl	.k_srsend	; normal device, return to KERNAL

	; calculate & save PCR bitmask for b4&5

	sei
	lda	bsour		; get transmit byte
	lsr			; shift b7-4 ..
	lsr
	lsr
	lsr			; .. into b3-0
	tax			; copy top nibble to .X
	lda	.pcrtab1,x	; lookup bitmask in table
	pha			; save mask for b4&5

	; calculate & save PCR bitmask for b6&7

	txa			; restore top nibble
	lsr			; shift b6&7 ..
	lsr			; .. to bottom of .A
	tax			; copy to .X
	lda	.pcrtab1,x 	; lookup bitmask in table
	sta	hi_nib		; save mask for b7&6
	
	lda	bsour		; get transmit byte
	and	#$0f		; mask off bottom nibble
	tax			; copy bottom nibble to .X

	lda	#%00000010	; serial DATA in bitmask

	; start of timing sensitive region
	
	; wait for serial DATA to go low
	
.wait_down:
	bit	via1_pa2	; {+4} test VIA 1 DRA
	beq	.wait_down	; {+2/3} loop until DATA is low

	lda	via2_pcr	; {+4} get VIA 2 PCR
	and	#%11011101	; {+2} mask off DATA & CLK out state
	sta	pcr_mask	; {+3} save PCR without DATA & CLK

	pha			; {+3} delay
	pla			; {+4} delay
	pha			; {+3} delay
	pla			; {+4} delay
	ifconst SYS_PAL
	cmp	(0,x)		; {+6} delay
	endif

	sta	via2_pcr	; {+4:32.3us} set VIA 2 PCR, CLK out high
	
	; send first pair of bits (b5&4) in DATA & CLK

	pla			; {+4} restore mask for b4&5
	ora	pcr_mask	; {+3} OR with rest of PCR state

	ifconst SYS_PAL
	nop			; {+2} delay
	endif

	sta	via2_pcr	; {+4:10.8us} set VIA 2 PCR

	; send second pair of bits (b7&6) in DATA & CLK

	lda	hi_nib		; {+3} get mask for b6b7
	ora	pcr_mask	; {+3} OR with rest of PCR state
	bit	0		; {+3} delay
	sta	via2_pcr	; {+4:12.7us} set VIA 2 PCR

	; send third pair of bits (b2&1) in DATA & CLK

	lda	.pcrtab2,x	; {+4} lookup bitmask in table
	ora	pcr_mask	; {+3} OR with rest of PCR state

	ifconst SYS_PAL
	nop			; {+2} delay
	endif

	sta	via2_pcr	; {+4:10.8us} set VIA 2 PCR

	; send fourth pair of bits (b3&0) in DATA & CLK

	lda	.pcrtab3,x	; {+4} lookup bitmask in table
	ora	pcr_mask	; {+3} OR with rest of PCR state
	nop			; {+2} delay
	sta	via2_pcr	; {+4:12.7us} set VIA 2 PCR

	ifconst SYS_PAL
	nop			; {+2} delay
	endif

	and	#%11011101	; {+2} mask off DATA & CLK out state
	bit	pcntr		; {+3} test for EOI
	bmi	.eoi		; {+2/3} EOI, don't bring data line low yet

	ora	#%00000010	; {+2} set serial CLK line low
.eoi:
	sta	via2_pcr	; {+4:12us} set VIA 2 PCR

	cmp	(0,x)		; {+6} delay
	ifconst SYS_PAL
	nop			; {+2} delay
	endif

	lda	pcr_mask	; {+3} get rest of PCR state
	ora	#%00000010	; {+2} set serial CLK line low
	sta	via2_pcr	; {+4:14.7us} set VIA 2 PCR

	; end of timing sensitive region

	lda	via1_pa2	; get VIA 1 DRA
	and	#%00000010	; mask serial DATA in bit
	bne	.timeout	; serial DATA low, timeout

	cli			; enable interrupts
	clc			; flag ok
	rts

.timeout:
	jmp	$eeb7		; set I/O status byte & complete command

	align	$10,$ff

	; mapping of b7-b4 to PCR bitmask
.pcrtab1:
	dc.b	$00,$02,$20,$22,$00,$02,$20,$22,$00,$02,$20,$22,$00,$02,$20,$22

	; mapping of b2 & b1 to PCR bitmask
.pcrtab2:
	dc.b	$00,$00,$20,$20,$00,$00,$20,$20,$02,$02,$22,$22,$02,$02,$22,$22

	; mapping of b3 & b0 to PCR bitmask
.pcrtab3:
	dc.b	$00,$20,$00,$20,$02,$22,$02,$22,$00,$20,$00,$20,$02,$22,$02,$22


/*******************************************************************************
 * Get byte from talking serial bus device                                     *
 *                                                                             *
 * Receive data into .A, STATUS will indicate EOI or errors.                   *
 * Unlike the KERNAL routine .X is *not* preserved.                            *
 *******************************************************************************/

	subroutine

.k_acptr:
	jmp	facptr		; use normal KERNAL routine

acptr:
	bit	jdev		; test JiffyDOS flag
	bpl	.k_acptr	; normal device, return to KERNAL

	sei			; disable interrupts

	; start of timing sensitive region

	; wait for either serial DATA or CLK to go low
.wait:
	lda	via1_pa2	; {+4} get VIA 1 DRA, no handshake
	and	#%00000011	; {+2} mask serial CLK and DATA in bits
	beq	.wait		; {+2/3} loop while both are high

	jsr	.ret		; {+12} delay
	ifconst SYS_PAL
	pha			; {+3} delay
	pla			; {+4} delay
	else
	nop			; {+2} delay
	nop			; {+2} delay
	endif

	lda	via2_pcr	; {+4} get VIA 2 PCR
	and	#%11011101	; {+2} clear DATA & CLK out state
	sta	via2_pcr	; {+4:38.1us} set DATA & CLK out high

	ora	#%00100000	; {+2} set serial DATA out
	tax			; {+2} copy PCR value to .X

	ifconst SYS_PAL
	bit	0		; {+3} delay
	else
	nop			; {+2} delay
	endif

	jsr	get_byte2	; {+6} receive using JiffyDOS byte mode

	; end of timing sensitive region

	sta	firt		; set received byte

	lda	via1_pa2	; get VIA 1 DRA, no handshake
	stx	via2_pcr	; set serial DATA out low

	ror			; shift CLK in into b7 ..
	ror			; .. and DATA in into Cb
	bmi	.eoi		; serial CLK high, EOI or timeout

.out:
	lda	firt		; get received byte
	cli			; enable interrupts
	clc			; flag ok
.ret:	rts

.eoi:
	bcs	.timeout	; serial DATA high, timeout
	lda	#%01000000	; set EOI bit
	jsr	oriost		; OR into I/O status byte
	bne	.out		; return received byte, branch always

.timeout:
	jmp	$ef40		; set I/O status byte & complete command


/*******************************************************************************
 * Get byte from serial bus using JiffyDOS protocol                            *
 *                                                                             *
 * Common routine used for both byte and sector mode, returns data in .A.      *
 *******************************************************************************/

	subroutine

get_byte:
	lda	pcr_mask	; {+3} get value with serial DATA out low
	sta	via2_pcr	; {+4:19.6us} set VIA 2 PCR

	lda	#%00000001	; {+2} serial CLK in bitmask
	bit	via1_pa2	; {+4} test VIA 1 DRA
	beq	.end_sector	; {+2/3} serial CLK in low, end of sector

	stx	via2_pcr	; {+4:11.7us} set serial DATA out high

get_byte2:
	lda	via1_pa2	; {11.7us:+4} get VIA 1 DRA with b1&0
	ror			; {+2} shift b0 into b7 ..
	ror			; {+2} .. and b1 into Cb
	and	#%10000000	; {+2} mask b0, b1 in Cb unchanged

	ifconst SYS_PAL
	nop			; {+2} delay
	endif

	ora	via1_pa2	; {9.8us:+4} OR VIA 1 DRA with b3&2

	rol			; {+2} shift b3-0 ..
	rol			; {+2} .. to correct position
	sta	lo_nib		; {+3} store lower nibble

	lda	via1_pa2	; {10.8us:+4} get VIA 1 DRA with b5&4
	ror			; {+2} shift b4 to b7 ..
	ror			; {+2} .. and b5 into Cb
	and	#%10000000	; {+2} mask b4, b5 in Cb unchanged

	ora	via1_pa2	; {9.8us:+4} OR VIA 1 DRA with b7&6

	; end of timing sensitive region

	ror			; shift b7-4 ..
	ror
	ror			; .. to correct position
	and	#$f0		; mask top four bits
	sta	hi_nib		; save high nibble
	lda	lo_nib		; get low nibble
	and	#$0f		; mask bottom four bits
	ora	hi_nib		; OR with top nibble
	rts

.end_sector:
	pla			; dump calling address low byte
	pla			; dump calling address high byte
	jmp	check_stop	; return to load routine
