;***********************************************************************************;
;***********************************************************************************;
;
; Driver for 16550 UART


;***********************************************************************************;
; public interface

UARTRGA	= $0293	; Virtual UART register A

		; bit	function
		; ---	--------
		;  7	unused
		;  6	unused
		; 543	parity
		; ---	------
		; xx0	disabled
		; 001	odd
		; 011	even
		; 101	mark
		; 111	space
		;  2	2 stop bits/1 stop bit
		; 10	word length
		; ---	-----------
		; 00	5 bits
		; 01	6 bits
		; 10	7 bits
		; 11	8 bits

UARTRGB	= $0294	; Virtual UART register B

		; bit	function
		; ---	--------
		;  7	hardware flow control
		;  6	unused
		;  5	unused
		;  4	unused
		; 3210	baud rate
		; ----	---------
		; 0000	   50
		; 0001	   75
		; 0010	  110
		; 0011	  134.5
		; 0100	  150
		; 0101	  300
		; 0110	  600
		; 0111	 1200
		; 1000	 2400
		; 1001	 3600
		; 1010	 4800
		; 1011	 9600
		; 1100  19200
		; 1101	38400
		; 1110	56000
		; 1111 128000

IO2BASE	= $9800
UARTBASE = IO2BASE+$0300

; 16550 UART registers

THR	= 0			; Transmiter Holding Register
RBR	= 0			; Receiver Buffer Register
DLL	= 0			; Divisor Latch LSB
DLM	= 1			; Divisor Latch MSB
IER	= 1			; Interrupt Enable Register
IIR	= 2			; Interrupt Identification Register
FCR	= 2			; FIFO Control Register
LCR	= 3			; Line Control Register
MCR	= 4			; MODEM Control Register
LSREG	= 5			; Line Status Register
MSR	= 6			; MODEM Status Register
SCR	= 7			; Scratch Register

FIFOSZ	= 16


MCRSAV	= $A7
RODBS	= $029D			; index to Tx buffer start


RSINIT:
	LDA #%00000011		; no parity, 8 bits
	STA UARTRGA
	LDA #%10001011		; hardware flow control, 9600 baud
	STA UARTRGB
	JMP RSOPEN		; Open RS-232 channel


;***********************************************************************************;
;
; open RS-232 channel
;
; Using the contents of the virtual UART registers set the baud rate, number of bits,
; stop bits and flow control lines.

RSOPEN:
	LDA #$00		; disable all interrupts
	STA UARTBASE+IER	; set Interrupt Enable
	LDA UARTRGB		; get virtual UART register B
	AND #%00001111		; mask the baud rate
	ASL			; multiply by two
	TAX			; index into baud rate table
	LDA #%10000000		; set DLAB
	STA UARTBASE+LCR	; enable divisor latches
	LDA BAUDTBL,X		; get divisor low byte
	STA UARTBASE+DLL	; set divisor latch low byte
	LDA BAUDTBL+1,X		; get divisor high byte
	STA UARTBASE+DLM	; set divisor latch high byte
	LDA UARTRGA		; get virtual UART register A
	AND #%00111111		; mask the word length, stop bits and parity
	STA UARTBASE+LCR	; set word length, stop bits and parity
	LDA #%00000001		; enable FIFOs
	STA UARTBASE+FCR	; set FIFO Control
	LDA #$00
	STA RODBS		; clear Tx buffer start
	LDA #%00001111		; raise DTR & RTS, OUT1 & OUT2 off
	STA MCRSAV
	STA UARTBASE+MCR	; set MODEM Control
	RTS


;***********************************************************************************;
;
; transmit a byte to RS-232 device
;
; Send the byte in .A. In poll mode this routine will block when the Tx FIFO is full.

	subroutine

RSSEND:
	PHA			; save byte

	; Poll mode 

.AGAIN	LDA UARTBASE+LSREG	; get Line Status
	AND #%00100000		; mask Tx Holding bit
	BEQ .TXNOTE		; branch if FIFO not empty
	LDA #$00		; reset counter
	STA RODBS

.TXNOTE	LDA RODBS		; get counter
	CMP #FIFOSZ		; compare with FIFO size
	BEQ .AGAIN		; loop while FIFO full

	LDA MCRSAV
	AND #%00001011		; OUT1 on
	STA MCRSAV
	STA UARTBASE+MCR
	INC RODBS		; increment counter
	PLA			; restore byte
	STA UARTBASE+THR	; store byte in FIFO
	LDA MCRSAV
	ORA #%00000100		; OUT1 off
	STA MCRSAV
	STA UARTBASE+MCR
	RTS


;***********************************************************************************;
;
; Baud rate divisors for 18.432 MHz crystal

DIV_BAUD50	= 23040
DIV_BAUD75	= 15360
DIV_BAUD110	= 10473
DIV_BAUD1345	= 8565
DIV_BAUD150	= 7680
DIV_BAUD300	= 3840
DIV_BAUD600	= 1920
DIV_BAUD1200	= 960		; datasheet reads 920
DIV_BAUD2400	= 480
DIV_BAUD3600	= 320
DIV_BAUD4800	= 240
DIV_BAUD9600	= 120
DIV_BAUD19200	= 60
DIV_BAUD38400	= 30
DIV_BAUD56000	= 21
DIV_BAUD128000	= 9


;***********************************************************************************;
;
; Baud rate table

BAUDTBL
	.WORD DIV_BAUD50
	.WORD DIV_BAUD75
	.WORD DIV_BAUD110
	.WORD DIV_BAUD1345
	.WORD DIV_BAUD150
	.WORD DIV_BAUD300
	.WORD DIV_BAUD600
	.WORD DIV_BAUD1200
	.WORD DIV_BAUD2400
	.WORD DIV_BAUD3600
	.WORD DIV_BAUD4800
	.WORD DIV_BAUD9600
	.WORD DIV_BAUD19200
	.WORD DIV_BAUD38400
	.WORD DIV_BAUD56000
	.WORD DIV_BAUD128000
